% Illustration of Taylor Estimation of Eigenvalue Shifts 
% Consider the matrix S= [k1+k2 -k2;-k2 k2]
% Assume we're interested in how the eigenvalues of S change as k1 varies
% We take the reference as k1=k2 =50
clear
S=[100 -50;-50 50];
% The derivative of the matrix with respect to k1
Sp=[1 0;0 0];
% The eigenvalues and right side eigenvectors in the reference state
[r,lam]=eig(S);
lam=diag(lam);
% left side eigenvectors
lt=inv(r);
% eigenvalues 1st derivative
FD=diag(lt*Sp*r);
% right eigenvector derivatives
f1p=pinv(S-eye(2)*lam(1))*(eye(2)*FD(1)-Sp)*r(:,1);
f2p=pinv(S-eye(2)*lam(2))*(eye(2)*FD(2)-Sp)*r(:,2);
Fp=[f1p f2p];
% derivative of left transpose
Lpt=-lt*Fp*lt;
% eigenvalue second derivative
SD=diag(Lpt*Sp*r+lt*Sp*Fp);

% We now change k1 from its initial value to 30 % damage and plot the exact
% vs the Taylor 1 term and two term approximations

kk=50:-1:25;
% Now assume that k1 changes from 50 to 45 
delta=kk-kk(1);
% the expected change is
change1=FD*(delta);
change2=FD*(delta)+SD*(delta).^2/2;
% so the new eigenvalues are
predict1=lam*ones(1,length(kk))+change1;
predict2=lam*ones(1,length(kk))+change2;
% compute the exact results
for j=1:length(kk)
Se=[kk(j)+50 -50;-50 50];
exact(:,j)=eig(Se);
end;
damage=(50-kk)/50*100;
plot(damage,predict1(1,:),'r',damage,predict2(1,:),'k',damage,exact(1,:),'b')
%subplot(2,1,2),plot(damage,predict1(2,:),damage,predict2(2,:),damage,exact(2,:),'g')
legend ('linear','second order','exact')
xlabel('% damage in k1')
ylabel('1st eigenvalue')