% "Simple Example on Sensitivity Enhancement in Open Loop".
clear
close all
% System
dof=1;
M=2;
K=200;
w=sqrt(K/M);
ccr=2*M*w;
z=0.02;
Cdam=z*ccr;
C1=1;

% In this example we selected the free parameters by first runing JD3 - the
% optimization yielded repeated eigenvalues for which the derivative is
% too large to be accceptable - we modified it by keeping the values for all the
% constants, except for ka (which was 4.0121) which we reduced to separate
% the eigenvalues until an attractive solution was obtained. 
% The sensitivity of the results to the parameters is an issue that requires examination.

% Free Parameters
ma=0.167;
ca=0.067;
ka=3.3;
La=6.16;
G2=78.4;

% Governing matrix
A=[0 1 0 0;-K/M -Cdam/M G2/M 0;0 0 0 1;-La*C1/ma 0 -ka/ma -ca/ma];

% Derivatives
Ap=zeros(4);
Ap(2,1)=-1/M;
[aa,bb]=eig(A);
Lam=diag(bb);
leftT=inv(aa);
der1=diag(leftT*Ap*aa);
for j=1:4;
    fp(:,j)=pinv(A-eye(4)*Lam(j))*(eye(4)*der1(j)-Ap)*aa(:,j);
end
flpT=-leftT*fp*leftT;
for j=1:4
    der2(j)=flpT(j,:)*Ap*aa(:,j)+leftT(j,:)*Ap*fp(:,j);
end;
der2=der2.';
% ****************************************
% Simulate (test)
% ****************************************
% No damage
Kd=K;
dt=0.001;
ns=120000;
t=(0:ns-1)*dt;
d0=0;
v0=0;
% Signal n(t)
w0a=27;
w0b=8;
w0c=15;
n=(sin(w0a*t)+sin(w0b*t)+sin(w0c*t));
[u]= Tma1(M,Cdam,Kd,dt,G2,n,d0,v0,ns-1,'lin',dt);

% Compute h(t);
ht=(-ma*(w0a^2*sin(w0a*t)+w0b^2*sin(w0b*t)+w0c^2*sin(w0c*t))+ca*(w0a*cos(w0a*t)+w0b*cos(w0b*t)+w0c*cos(w0c*t))+ka*n)+La*u';
% Identify
E9=ht';
zz=[u;n'];
[Aid]=dsr9999(zz,E9,10);
vv=logm(Aid)/dt;
LampID_U=eig(vv);

% ****************************************
% Simulate with damage (test)
% ****************************************

percent_damage=5;
Kd=K*(1-percent_damage/100);
[ud]= Tma1(M,Cdam,Kd,dt,G2,n,d0,v0,ns-1,'lin',dt);
% Compute h(t);
htd=(-ma*(w0a^2*sin(w0a*t)+w0b^2*sin(w0b*t)+w0c^2*sin(w0c*t))+ca*(w0a*cos(w0a*t)+w0b*cos(w0b*t)+w0c*cos(w0c*t))+ka*n)+La*ud';
% Identify
E9=htd';
zz=[ud;n'];
[Aid]=dsr9999(zz,E9,10);
vv=logm(Aid)/dt;
LampID_D=eig(vv);

% Expected eigenvalues using a second order approximation
LampD_expected=Lam+der1*(Kd-K)+0.5*der2*(Kd-K)^2;
LampU_expected=Lam;

% If we operated in standard fashion the eigenvalue in the undamaged and
% damaged states are
omega1=sqrt(K/M);
StandU=-omega1*z+omega1*sqrt(1-z^2)*1i;
omega2=sqrt(Kd/M);
StandD=-omega2*z+omega2*sqrt(1-z^2)*1i;

% Summary of results
format short g
StandU
StandD
LampID_U
LampID_D
LampU_expected
LampD_expected