function [resdVec] = QuadEigResiduals(x, M, lambda, CvTilde, n_modes)
% function [resdVec] = QuadEigResiduals(x, M, lambda, CvTilde, n_modes)
%
% Input
% x: the optimization variable, containing unknown stiffness and
%    damping values
% M: known mass matrix
% lambda: a vector containing lambda_r
% CvTilde: a matrix whose columns are {Ctilde * v_r_tilde}
% n_modes: number of modes that will be used to evaluate the residual
%   vector
%
% Output
% resdVec: the residual vector required by lsqnonlin, containing real and
%    imaginary parts of n_modes number of residual vectors.
%
k1 = x(1); k2 = x(2); k3 = x(3); k4 = x(4);

K = [ k1+k2,   -k2,     0,   0; ...
    -k2, k2+k3,   -k3,   0; ...
    0,   -k3, k3+k4, -k4; ...
    0,     0,   -k4,  k4];

c1 = x(5); c2 = x(6); c3 = x(7); c4 = x(8);

C = [  c1+c2,   -c2,     0,   0; ...
    -c2, c2+c3,   -c3,   0; ...
    0,   -c3, c3+c4, -c4; ...
    0,     0,   -c4,  c4];

N = size(M, 1);
nx = 2 * N;
resdVec = zeros(nx * n_modes, 1);
for r = 1 : n_modes
    resdVec((r-1) * nx + 1 : r * nx, 1) = ...
        [ real( (K + lambda(r) * C + lambda(r)^2 * M) * CvTilde(:,r) );
          imag( (K + lambda(r) * C + lambda(r)^2 * M) * CvTilde(:,r) ) ];
end
